package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Combobox;
import org.zkoss.zul.Comboitem;
import org.zkoss.zul.Paging;
import org.zkoss.zul.api.Datebox;

import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.Feriado;
import com.rjconsultores.ventaboletos.service.EmpresaService;
import com.rjconsultores.ventaboletos.service.EstadoService;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.MyTextbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderFeriado;

@Controller("busquedaConfigFeriadoController")
@Scope("prototype")
public class BusquedaConfigFeriadoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = 1L;
	@Autowired
	private transient PagedListWrapper<Feriado> plwFeriado;
	@Autowired
	private EmpresaService empresaService;
	@Autowired
	private EstadoService estadoService;
	private List<Empresa> lsEmpresa;
	private List<Estado> lsEstado;
	private MyListbox configFeriadoList;
	private Paging pagingConfigFeriado;
	private MyTextbox textboxDescricao;
	private Datebox fecFeriado;
	private Combobox cmbEstado;
	private Combobox cmbEmpresa;

	public List<Empresa> getLsEmpresa() {
		return lsEmpresa;
	}

	public void setLsEmpresa(List<Empresa> lsEmpresa) {
		this.lsEmpresa = lsEmpresa;
	}

	public List<Estado> getLsEstado() {
		return lsEstado;
	}

	public void setLsEstado(List<Estado> lsEstado) {
		this.lsEstado = lsEstado;
	}

	@Override
	public void doAfterCompose(Component comp) throws Exception {
		lsEmpresa = empresaService.obtenerTodos();
		lsEstado = estadoService.obtenerTodos();
		super.doAfterCompose(comp);

		configFeriadoList.setItemRenderer(new RenderFeriado());
		configFeriadoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				Feriado c = (Feriado) configFeriadoList.getSelected();
				verFeriado(c);
			}
		});

		refreshLista();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verFeriado(Feriado a) {
		if (a == null) {
			return;
		}

		Map args = new HashMap();
		args.put("feriado", a);
		args.put("configFeriadoList", configFeriadoList);

		openWindow("/gui/configuraciones_comerciales/editarConfigFeriado.zul",
				Labels.getLabel("busquedaConfigFeriadoController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<Feriado> searchFeriado =
				new HibernateSearchObject<Feriado>(Feriado.class, pagingConfigFeriado.getPageSize());

		Comboitem cbiEmpresa = cmbEmpresa.getSelectedItem();
		if (cbiEmpresa != null) {
			Empresa empresa = (Empresa) cbiEmpresa.getValue();
			searchFeriado.addFilterEqual("empresa", empresa);
		}

		Comboitem cbiEstado = cmbEstado.getSelectedItem();
		if (cbiEstado != null) {
			Estado estado = (Estado) cbiEstado.getValue();
			searchFeriado.addFilterEqual("estado", estado);
		}

		String descricao = textboxDescricao.getText();
		if ((descricao != null)) {
			searchFeriado.addFilterLike("descferiado", "%" + descricao.trim().concat("%"));
		}

		if (fecFeriado.getValue() != null) {
			Date data = fecFeriado.getValue();
			GregorianCalendar gc = new GregorianCalendar();
			gc.setTime(data);
			gc.set(Calendar.HOUR_OF_DAY, 00);
			gc.set(Calendar.MINUTE, 00);
			gc.set(Calendar.SECOND, 00);

			searchFeriado.addFilterEqual("fecferiado", gc.getTime());
		}

		searchFeriado.addFilterEqual(ACTIVO, Boolean.TRUE);
		searchFeriado.addSortAsc("descferiado");
		searchFeriado.addFilterNotEqual("feriadoId", -1);

		plwFeriado.init(searchFeriado, configFeriadoList, pagingConfigFeriado);
		if (configFeriadoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaAlertaCtrlController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}
	}

	public void onClick$btnPesquisa(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}

	public void onClick$btnNovo(Event ev) {
		verFeriado(new Feriado());
	}
}
