package com.rjconsultores.ventaboletos.web.gui.controladores.configuracioneccomerciales;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Controller;
import org.zkoss.util.resource.Labels;
import org.zkoss.zhtml.Filedownload;
import org.zkoss.zhtml.Messagebox;
import org.zkoss.zk.ui.Component;
import org.zkoss.zk.ui.event.Event;
import org.zkoss.zk.ui.event.EventListener;
import org.zkoss.zul.Button;
import org.zkoss.zul.Paging;
import org.zkoss.zul.Textbox;

import com.rjconsultores.ventaboletos.entidad.GrupoContrato;
import com.rjconsultores.ventaboletos.service.MudancaMassivaContratoService;
import com.rjconsultores.ventaboletos.utilerias.DateUtil;
import com.rjconsultores.ventaboletos.vo.configuracioneccomerciales.TarifaGrupoContratoVO;
import com.rjconsultores.ventaboletos.web.utilerias.MyGenericForwardComposer;
import com.rjconsultores.ventaboletos.web.utilerias.MyListbox;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.HibernateSearchObject;
import com.rjconsultores.ventaboletos.web.utilerias.paginacion.PagedListWrapper;
import com.rjconsultores.ventaboletos.web.utilerias.render.RenderPadrao;

@Controller("busquedaGrupoContratoController")
@Scope("prototype")
public class BusquedaGrupoContratoController extends MyGenericForwardComposer {

	private static final long serialVersionUID = -670882588234850910L;
	@Autowired
	private transient PagedListWrapper<GrupoContrato> plwpagingGrupoContrato;
	@Autowired
	private MudancaMassivaContratoService mudancaMassivaService;
	
	private MyListbox grupoContratoList;
	private Paging pagingGrupoContrato;
	private Textbox txtDescricao;
	private Button btnExportar;
	
	@Override
	public void doAfterCompose(Component comp) throws Exception {
		super.doAfterCompose(comp);

		grupoContratoList.setItemRenderer(new RenderPadrao<GrupoContrato>(GrupoContrato.class));
		grupoContratoList.addEventListener("onDoubleClick", new EventListener() {

			@Override
			public void onEvent(Event event) throws Exception {
				GrupoContrato cc = (GrupoContrato) grupoContratoList.getSelected();
				verGrupoContrato(cc);
			}
		});

		refreshLista();

		txtDescricao.focus();
	}

	@SuppressWarnings({ "rawtypes", "unchecked" })
	private void verGrupoContrato(GrupoContrato gc) {
		if (gc == null) {
			return;
		}

		Map args = new HashMap();
		args.put("grupoContrato", gc);
		args.put("grupoContratoList", grupoContratoList);

		openWindow("/gui/configuraciones_comerciales/negcorporativos/editarGrupoContrato.zul",
				Labels.getLabel("editarGrupoContratoController.window.title"), args, MODAL);
	}

	private void refreshLista() {
		HibernateSearchObject<GrupoContrato> configCategoriaBusqueda =new HibernateSearchObject<GrupoContrato>(GrupoContrato.class,pagingGrupoContrato.getPageSize());

		if (txtDescricao.getText().length() > 0) {
			configCategoriaBusqueda.addFilterLike("nomeGrupoContrato", "%" + txtDescricao.getText().trim().concat("%"));
		}
		
		configCategoriaBusqueda.addFilterEqual(ACTIVO, Boolean.TRUE);
		configCategoriaBusqueda.addSortAsc("nomeGrupoContrato");

		plwpagingGrupoContrato.init(configCategoriaBusqueda, grupoContratoList, pagingGrupoContrato);

		if (grupoContratoList.getData().length == 0) {
			try {
				Messagebox.show(Labels.getLabel("MSG.ningunRegistro"),
						Labels.getLabel("busquedaGrupoContratoController.window.title"),
						Messagebox.OK, Messagebox.INFORMATION);
			} catch (InterruptedException ex) {
			}
		}else {
			btnExportar.setVisible(true);
		}
	}

	public void onClick$btnPesquisar(Event ev) {
		refreshLista();
	}

	public void onClick$btnRefresh(Event ev) {
		refreshLista();
	}
	
	public void onClick$btnExportar(Event ev) {
		exportarCvs();
	}

	public void onClick$btnNovo(Event ev) {
		verGrupoContrato(new GrupoContrato());
	}
	
	private void exportarCvs() {
		StringBuilder resultado = new StringBuilder("ID;Grupo de Contrato;Origem;DescOrigem;Destino;DescDestino;TARIFA NOVA;Tarifa Atual\r\n");
		for (Object item : grupoContratoList.getListData()) {
			GrupoContrato obj = (GrupoContrato)item;
			List<TarifaGrupoContratoVO> tarifasContrato = mudancaMassivaService.buscarTarifasGrupo(obj.getGrupoContratoId());
			for (TarifaGrupoContratoVO vo : tarifasContrato) {
				resultado.append(vo.toCsv());
			}
		}
		
		InputStream is = new ByteArrayInputStream(resultado.toString().getBytes());
		String data = DateUtil.getStringDate24Hour(Calendar.getInstance().getTime());
		Filedownload.save(is, "application/txt", "modificacao_massiva"+data+".csv");
		
	}
}
